"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Formatter = void 0;
var _jsStringWrap = require("../utils/jsStringWrap");
var _utils = require("../utils");
var _utils2 = require("../playwright/utils");
/**
 * Helper to format Playwright test file.
 */

const supportsTags = _utils2.playwrightVersion >= '1.42.0';
class Formatter {
  config;
  constructor(config) {
    this.config = config;
  }
  fileHeader(featureUri, importTestFrom) {
    // always use "/" for imports, see #91
    const importTestFromFile = (0, _utils.toPosixPath)((importTestFrom === null || importTestFrom === void 0 ? void 0 : importTestFrom.file) || '@zohodesk/testinglibrary');
    let varName = (importTestFrom === null || importTestFrom === void 0 ? void 0 : importTestFrom.varName) || 'test';
    if (varName !== 'test') {
      varName = `${varName} as test`;
    }
    return [`/** Generated from: ${featureUri} */`,
    // prettier-ignore
    // this.quoted() is not possible for 'import from' as backticks not parsed
    `import { ${varName} } from ${JSON.stringify(importTestFromFile)};`, ''];
  }
  describe(node, children) {
    const firstLine = `test.${this.getFunction('describe', node)}(${this.quoted(node.title)}, () => {`;
    if (!children.length) {
      return [`${firstLine}});`, ''];
    }
    return [firstLine,
    // prettier-ignore
    ...this.describeConfigure(node).map(indent), '', ...children.map(indent), `});`, ''];
  }
  beforeEach(fixtures, children) {
    const fixturesStr = [...fixtures].join(', ');
    // prettier-ignore
    return [`test.beforeEach(async ({ ${fixturesStr} }) => {`, ...children.map(indent), `});`, ''];
  }
  test(node, fixtures, children) {
    const fixturesStr = [...fixtures].join(', ');
    const title = this.quoted(node.title);
    const tags = this.testTags(node);
    const firstLine = `${this.getFunction('test', node)}(${title}, ${tags}async ({ ${fixturesStr} }) => {`;
    if (!children.length) {
      return [`${firstLine}});`, ''];
    }
    const lines = [firstLine,
    // prettier-ignore
    ...children.map(indent), `});`, ''];
    // wrap test into anonymous describe in case of retries / timeout tags
    const specialTagsConfigure = this.describeConfigure(node);
    return specialTagsConfigure.length ? ['test.describe(() => {',
    // prettier-ignore
    ...specialTagsConfigure.map(indent), '', ...lines.map(indent), `});`, ''] : lines;
  }
  // eslint-disable-next-line max-params
  step(keyword, text, argument, fixtureNames = []) {
    const fixtures = fixtureNames.length ? `{ ${fixtureNames.join(', ')} }` : '';
    const argumentArg = argument ? JSON.stringify(argument) : fixtures ? 'null' : '';
    const textArg = this.quoted(text);
    const args = [textArg, argumentArg, fixtures].filter(arg => arg !== '').join(', ');
    return `await ${keyword}(${args});`;
  }
  missingStep(keyword, text) {
    return `// missing step: ${keyword}(${this.quoted(text)});`;
  }
  technicalSection(testMetaBuilder, featureUri, fixtures) {
    return ['// == technical section ==',
    // prettier-ignore
    '', 'test.use({', ...['$test: ({}, use) => use(test),', '$testMetaMap: ({}, use) => use(testMetaMap),', `$uri: ({}, use) => use(${this.quoted(featureUri)}),`, ...fixtures].map(indent), '});', '', 'const testMetaMap = {', ...testMetaBuilder.getObjectLines().map(indent), '};'];
  }
  bddWorldFixtures() {
    const fixturesObj = {
      page: null,
      context: null,
      browser: null,
      browserName: null,
      request: null
    };
    const fixtures = Object.keys(fixturesObj).join(', ');
    return [`$bddWorldFixtures: ({ ${fixtures} }, use) => use({ ${fixtures} }),`];
  }
  scenarioHookFixtures(fixtureNames) {
    if (!fixtureNames.length) {
      return [];
    }
    const fixtures = fixtureNames.join(', ');
    return [`$scenarioHookFixtures: ({ ${fixtures} }, use) => use({ ${fixtures} }),`];
  }
  workerHookFixtures(fixtureNames) {
    if (!fixtureNames.length) {
      return [];
    }
    const fixtures = fixtureNames.join(', ');
    const scope = this.quoted('worker');
    return [`$workerHookFixtures: [({ ${fixtures} }, use) => use({ ${fixtures} }), { scope: ${scope} }],`];
  }
  langFixture(lang) {
    return [`$lang: ({}, use) => use(${this.quoted(lang)}),`];
  }
  // eslint-disable-next-line complexity
  getFunction(baseFn, node) {
    if (node.specialTags.only) {
      return `${baseFn}.only`;
    }
    // describe.fail is not supported
    if (baseFn === 'test' && node.specialTags.fail) {
      return `${baseFn}.fail`;
    }
    if (node.specialTags.skip) {
      return `${baseFn}.skip`;
    }
    if (node.specialTags.fixme) {
      return `${baseFn}.fixme`;
    }
    return baseFn;
  }
  describeConfigure(node) {
    const options = {};
    const {
      retries,
      timeout,
      mode
    } = node.specialTags;
    if (retries !== undefined) {
      options.retries = retries;
    }
    if (timeout !== undefined) {
      options.timeout = timeout;
    }
    if (mode !== undefined) {
      options.mode = mode;
    }
    return Object.keys(options).length ? [`test.describe.configure(${JSON.stringify(options)});`] : [];
  }
  testTags(node) {
    return supportsTags && node.tags.length ? `{ tag: [${node.tags.map(tag => this.quoted(tag)).join(', ')}] }, ` : '';
  }
  testTimeout(node) {
    const {
      timeout
    } = node.specialTags;
    return timeout !== undefined ? [`test.setTimeout(${timeout});`] : [];
  }
  /**
     * Apply this function only to string literals (mostly titles here).
     * Objects and arrays are handled with JSON.strinigfy,
     * b/c object keys can't be in backticks.
     * See: https://stackoverflow.com/questions/33194138/template-string-as-object-property-name
     */
  quoted(str) {
    return (0, _jsStringWrap.jsStringWrap)(str, {
      quotes: this.config.quotes
    });
  }
}
exports.Formatter = Formatter;
function indent(value) {
  return value ? `${'  '}${value}` : value;
}