"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TestFilesGenerator = void 0;
var _promises = _interopRequireDefault(require("fs/promises"));
var _path = _interopRequireDefault(require("path"));
var _fastGlob = _interopRequireDefault(require("fast-glob"));
var _testFile = require("./testFile");
var _loadConfig = require("../cucumber/loadConfig");
var _loadFeatures = require("../cucumber/loadFeatures");
var _loadSteps = require("../cucumber/loadSteps");
var _config = require("../config");
var _snippets = require("../snippets");
var _steps = require("../steps/decorators/steps");
var _transform = require("../playwright/transform");
var _configDir = require("../config/configDir");
var _logger = require("../utils/logger");
var _tagExpressions = _interopRequireDefault(require("@cucumber/tag-expressions"));
var _exit = require("../utils/exit");
var _createBdd = require("../steps/createBdd");
var _resolveFeaturePaths = require("../cucumber/resolveFeaturePaths");
/**
 * Generate playwright test files from Gherkin documents.
 */

class TestFilesGenerator {
  config;
  // all these props are exist
  runConfiguration;
  featuresLoader = new _loadFeatures.FeaturesLoader();
  supportCodeLibrary;
  files = [];
  tagsExpression;
  logger;
  constructor(config) {
    this.config = config;
    this.logger = new _logger.Logger({
      verbose: config.verbose
    });
    if (config.tags) {
      this.tagsExpression = (0, _tagExpressions.default)(config.tags);
    }
  }
  async generate() {
    await (0, _exit.withExitHandler)(async () => {
      await this.loadCucumberConfig();
      await Promise.all([this.loadFeatures(), this.loadSteps()]);
      this.buildFiles();
      await this.checkUndefinedSteps();
      this.checkImportTestFrom();
      await this.clearOutputDir();
      await this.saveFiles();
    });
  }
  async extractSteps() {
    await this.loadCucumberConfig();
    await this.loadSteps();
    return this.supportCodeLibrary.stepDefinitions;
  }
  // todo: combine with extractSteps
  async extractUnusedSteps() {
    await this.loadCucumberConfig();
    await Promise.all([this.loadFeatures(), this.loadSteps()]);
    this.buildFiles();
    return this.supportCodeLibrary.stepDefinitions.filter(stepDefinition => {
      const isUsed = this.files.some(file => file.usedStepDefinitions.has(stepDefinition));
      return !isUsed;
    });
  }
  async loadCucumberConfig() {
    const environment = {
      cwd: (0, _configDir.getPlaywrightConfigDir)()
    };
    const {
      runConfiguration
    } = await (0, _loadConfig.loadConfig)({
      provided: (0, _config.extractCucumberConfig)(this.config)
    }, environment);
    this.runConfiguration = runConfiguration;
    this.warnForTsNodeRegister();
  }
  async loadFeatures() {
    const cwd = (0, _configDir.getPlaywrightConfigDir)();
    const {
      defaultDialect
    } = this.runConfiguration.sources;
    const {
      featurePaths
    } = await (0, _resolveFeaturePaths.resovleFeaturePaths)(this.runConfiguration, {
      cwd
    });
    this.logger.log(`Loading features from paths (${featurePaths.length}):`);
    featurePaths.forEach(featurePath => this.logger.log(featurePath));
    if (featurePaths.length) {
      await this.featuresLoader.load(featurePaths, {
        relativeTo: cwd,
        defaultDialect
      });
      this.handleParseErrors();
    }
    this.logger.log(`Loaded features: ${this.featuresLoader.getDocumentsCount()}`);
  }
  async loadSteps() {
    const {
      requirePaths,
      importPaths
    } = this.runConfiguration.support;
    this.logger.log(`Loading steps from: ${requirePaths.concat(importPaths).join(', ')}`);
    const environment = {
      cwd: (0, _configDir.getPlaywrightConfigDir)()
    };
    this.supportCodeLibrary = await (0, _loadSteps.loadSteps)(this.runConfiguration, environment);
    await this.loadDecoratorSteps();
    this.logger.log(`Loaded steps: ${this.supportCodeLibrary.stepDefinitions.length}`);
  }
  async loadDecoratorSteps() {
    const {
      importTestFrom
    } = this.config;
    if (importTestFrom) {
      // require importTestFrom for case when it is not required by step definitions
      // possible re-require but it's not a problem as it is cached by Node.js
      await (0, _transform.requireTransform)().requireOrImport(importTestFrom.file);
      (0, _steps.appendDecoratorSteps)(this.supportCodeLibrary);
    }
  }
  buildFiles() {
    this.files = this.featuresLoader.getDocumentsWithPickles().map(gherkinDocument => {
      return new _testFile.TestFile({
        gherkinDocument,
        supportCodeLibrary: this.supportCodeLibrary,
        // doc.uri is always relative to cwd (coming after cucumber handling)
        // see: https://github.com/cucumber/cucumber-js/blob/main/src/api/gherkin.ts#L51
        outputPath: this.getSpecPathByFeaturePath(gherkinDocument.uri),
        config: this.config,
        tagsExpression: this.tagsExpression
      }).build();
    }).filter(file => file.testCount > 0);
  }
  getSpecPathByFeaturePath(relFeaturePath) {
    const configDir = (0, _configDir.getPlaywrightConfigDir)();
    const absFeaturePath = _path.default.resolve(configDir, relFeaturePath);
    const relOutputPath = _path.default.relative(this.config.featuresRoot, absFeaturePath);
    if (relOutputPath.startsWith('..')) {
      (0, _exit.exit)(`All feature files should be located underneath featuresRoot.`, `Please change featuresRoot or paths in configuration.\n`, `featureFile: ${absFeaturePath}\n`, `featuresRoot: ${this.config.featuresRoot}\n`);
    }
    const absOutputPath = _path.default.resolve(this.config.outputDir, relOutputPath);
    return `${absOutputPath}.spec.js`;
  }
  async checkUndefinedSteps() {
    const undefinedSteps = this.files.reduce((sum, file) => sum + file.undefinedSteps.length, 0);
    if (undefinedSteps > 0) {
      const snippets = new _snippets.Snippets(this.files, this.runConfiguration, this.supportCodeLibrary);
      await snippets.print();
      (0, _exit.exit)();
    }
  }
  checkImportTestFrom() {
    if (_createBdd.hasCustomTest && !this.config.importTestFrom) {
      (0, _exit.exit)(`When using custom "test" function in createBdd() you should`, `set "importTestFrom" config option that points to file exporting custom test.`);
    }
  }
  async saveFiles() {
    this.files.forEach(file => {
      file.save();
      this.logger.log(`Generated: ${_path.default.relative(process.cwd(), file.outputPath)}`);
    });
    this.logger.log(`Generated files: ${this.files.length}`);
  }
  async clearOutputDir() {
    const pattern = `${_fastGlob.default.convertPathToPattern(this.config.outputDir)}/**/*.spec.js`;
    const testFiles = await (0, _fastGlob.default)(pattern);
    this.logger.log(`Clearing output dir: ${testFiles.length} file(s)`);
    const tasks = testFiles.map(testFile => _promises.default.rm(testFile));
    await Promise.all(tasks);
  }
  warnForTsNodeRegister() {
    if ((0, _loadSteps.hasTsNodeRegister)(this.runConfiguration)) {
      this.logger.warn(`WARNING: usage of requireModule: ['ts-node/register'] is not recommended for playwright-bdd.`, `Remove this option from defineBddConfig() and`, `Playwright's built-in loader will be used to compile TypeScript step definitions.`);
    }
  }
  handleParseErrors() {
    const {
      parseErrors
    } = this.featuresLoader;
    if (parseErrors.length) {
      const message = parseErrors.map(parseError => {
        return `Parse error in "${parseError.source.uri}" ${parseError.message}`;
      }).join('\n');
      (0, _exit.exit)(message);
    }
  }
}
exports.TestFilesGenerator = TestFilesGenerator;