"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getLocationByOffset = getLocationByOffset;
exports.getLocationInFile = getLocationInFile;
var _url = _interopRequireDefault(require("url"));
var _utils = require("./utils");
/**
 * Finds first frame in the callstack that matches provided file.
 * Constructs location object from that frame.
 *
 * Example:
 * Calling getLocationInFile('file-3.js');
 * Call stack:
 * - at <anonymous> (file-1.js:1:2)
 * - at myFunction2 (file-2.js:3:4)
 * - at myFunction3 (file-3.js:5:6)
 * - at myFunction4 (file-4.js:7:8)
 *
 * Returned value: { file: 'file-3.js', line: 5, column: 6 }
 */
function getLocationInFile(filePath) {
  const filePathUrl = _url.default.pathToFileURL(filePath).toString();
  return getLocationBy(stackFrames => {
    return stackFrames.find(frame => {
      const frameFile = frame.getFileName();
      return frameFile === filePath || frameFile === filePathUrl;
    });
  });
}
function getLocationByOffset(offset) {
  return getLocationBy(stackFrames => stackFrames[offset]);
}
/**
 * Iterates stacktrace of the call of this function
 * and finds frame using provided findFrame predicate.
 * Constructs location object from the found frame.
 *
 * This function is based on Playwright's wrapFunctionWithLocation().
 * See: https://github.com/microsoft/playwright/blob/main/packages/playwright/src/transform/transform.ts#L261
 */
function getLocationBy(findFrame) {
  const {
    sourceMapSupport
  } = (0, _utils.requirePlaywrightModule)('lib/utilsBundle.js');
  const oldPrepareStackTrace = Error.prepareStackTrace;
  // modify prepareStackTrace to return Location object instead of string
  Error.prepareStackTrace = (error, stackFrames) => {
    const foundFrame = findFrame(stackFrames);
    if (!foundFrame) {
      return {
        file: '',
        line: 0,
        column: 0
      };
    }
    const frame = sourceMapSupport.wrapCallSite(foundFrame);
    const fileName = frame.getFileName();
    // Node error stacks for modules use file:// urls instead of paths.
    const file = fileName !== null && fileName !== void 0 && fileName.startsWith('file://') ? _url.default.fileURLToPath(fileName) : fileName;
    return {
      file,
      line: frame.getLineNumber(),
      column: frame.getColumnNumber()
    };
  };
  // commented stackTraceLImit modification, todo: check if it has perf impact
  // const oldStackTraceLimit = Error.stackTraceLimit;
  // Error.stackTraceLimit = level + 1;
  const obj = {};
  Error.captureStackTrace(obj);
  const location = obj.stack;
  // Error.stackTraceLimit = oldStackTraceLimit;
  Error.prepareStackTrace = oldPrepareStackTrace;
  return location;
}