"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AttachmentMapper = void 0;
var _AutofillMap = require("../../../utils/AutofillMap");
var _pwStepUtils = require("./pwStepUtils");
var _stripAnsiEscapes = require("../../../utils/stripAnsiEscapes");
class AttachmentMapper {
  result;
  stepAttachments = new _AutofillMap.AutofillMap();
  unusedAttachments = [];
  constructor(result) {
    this.result = result;
    this.mapAttachments();
  }
  getStepAttachments(pwStep) {
    return this.stepAttachments.get(pwStep) || [];
  }
  mapAttachments() {
    const allAttachments = this.result.attachments.slice();
    const attachmentSteps = (0, _pwStepUtils.collectStepsWithCategory)(this.result, 'attach');
    attachmentSteps.forEach(attachmentStep => {
      this.mapAttachment(attachmentStep, allAttachments);
    });
    this.unusedAttachments.push(...allAttachments);
    this.mapUnusedAttachments();
    this.mapStdoutAttachments('stdout');
    this.mapStdoutAttachments('stderr');
  }
  mapAttachment(attachmentStep, allAttachments) {
    const index = allAttachments.findIndex(a => getAttachmentStepTitle(a.name) === attachmentStep.title);
    if (index === -1) {
      throw new Error(`Attachment not found for step: ${attachmentStep.title}`);
    }
    const [foundAttachment] = allAttachments.splice(index, 1);
    const parentStep = attachmentStep.parent;
    // step.parent is empty:
    // - in PW = 1.34 for screenshot attachment
    // - in PW <= 1.40 when testInfo.attach() promise
    // is awaited in the next async tick: 'attach' steps are on the top level
    const stepAttachments = parentStep ? this.stepAttachments.getOrCreate(parentStep, () => []) : this.unusedAttachments;
    stepAttachments.push(foundAttachment);
  }
  mapUnusedAttachments() {
    if (!this.unusedAttachments.length) {
      return;
    }
    // map unused attachments to the 'After Hooks' step
    const afterHooksRoot = this.getAfterHooksRoot();
    const stepAttachments = this.stepAttachments.getOrCreate(afterHooksRoot, () => []);
    stepAttachments.push(...this.unusedAttachments);
  }
  mapStdoutAttachments(name) {
    var _this$result$name;
    // map stdout / stderr to the 'After Hooks' step
    if (!((_this$result$name = this.result[name]) !== null && _this$result$name !== void 0 && _this$result$name.length)) {
      return;
    }
    const body = this.result[name].map(s => typeof s === 'string' ? (0, _stripAnsiEscapes.stripAnsiEscapes)(s) : s).join('');
    const afterHooksRoot = this.getAfterHooksRoot();
    const stepAttachments = this.stepAttachments.getOrCreate(afterHooksRoot, () => []);
    stepAttachments.push({
      name,
      contentType: 'text/plain',
      body: Buffer.from(body)
    });
  }
  getAfterHooksRoot() {
    const afterHooksRoot = (0, _pwStepUtils.getHooksRootPwStep)(this.result, 'after');
    if (!afterHooksRoot) {
      throw new Error(`Can not find after hooks root.`);
    }
    return afterHooksRoot;
  }
}
// See: https://github.com/microsoft/playwright/blob/main/packages/playwright/src/worker/testInfo.ts#L413
exports.AttachmentMapper = AttachmentMapper;
function getAttachmentStepTitle(attachmentName) {
  return `attach "${attachmentName}"`;
}