"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MessagesBuilder = void 0;
var _TestCaseRun = require("./TestCaseRun");
var _TestCase = require("./TestCase");
var _Meta = require("./Meta");
var _timing = require("./timing");
var _events = _interopRequireDefault(require("events"));
var _EventDataCollector = _interopRequireDefault(require("../../../cucumber/formatter/EventDataCollector"));
var _AutofillMap = require("../../../utils/AutofillMap");
var _GherkinDocuments = require("./GherkinDocuments");
var _Pickles = require("./Pickles");
class MessagesBuilder {
  report = {
    meta: null,
    source: [],
    gherkinDocument: [],
    pickle: [],
    stepDefinition: [],
    hook: [],
    testRunStarted: null,
    testCase: [],
    testCaseRuns: [],
    testRunFinished: null
  };
  fullResult;
  onTestEnds = [];
  testCaseRuns = [];
  testCases = new _AutofillMap.AutofillMap();
  hooks = new _AutofillMap.AutofillMap();
  gherkinDocuments = new _GherkinDocuments.GherkinDocuments();
  fullResultTiming;
  onEndPromise;
  onEndPromiseResolve = () => {};
  buildMessagesPromise;
  eventDataCollectorEmitter = new _events.default();
  eventDataCollector = new _EventDataCollector.default(this.eventDataCollectorEmitter);
  constructor() {
    this.onEndPromise = new Promise(resolve => this.onEndPromiseResolve = resolve);
  }
  onTestEnd(test, result) {
    // For skipped tests Playwright doesn't run fixtures
    // and we don't have bddData attachment -> don't know feature uri.
    // Don't add such test run to report.
    if (test.expectedStatus === 'skipped') {
      return;
    }
    // Important to create TestCaseRun here,
    // b/c test properties can change after retries (e.g. annotations)
    const testCaseRun = new _TestCaseRun.TestCaseRun(test, result, this.hooks);
    this.testCaseRuns.push(testCaseRun);
  }
  onEnd(fullResult) {
    this.fullResult = fullResult;
    this.onEndPromiseResolve();
  }
  /**
     * Builds Cucumber messages.
     * Note: wrapped into promise to build messages once for all reporters.
     */
  async buildMessages() {
    if (!this.buildMessagesPromise) {
      this.buildMessagesPromise = this.doBuildMessages();
    }
    return this.buildMessagesPromise;
  }
  // eslint-disable-next-line max-statements
  async doBuildMessages() {
    await this.onEndPromise;
    // order here is important
    // this.createTestCaseRuns();
    await this.loadFeatures();
    this.createTestCases();
    this.addMeta();
    this.addSourcesAndDocuments();
    this.addPickles();
    this.addHooks();
    this.addTestRunStarted();
    this.addTestCases();
    this.addTestCaseRuns();
    this.addTestRunFinished();
    this.buildEventDataCollector();
  }
  emitMessages(eventBroadcaster) {
    Object.values(this.report).forEach(value => {
      if (!value) {
        return;
      }
      const messages = Array.isArray(value) ? value : [value];
      messages.forEach(message => eventBroadcaster.emit('envelope', message));
    });
  }
  createTestCases() {
    this.testCaseRuns.forEach(testCaseRun => {
      const testId = testCaseRun.test.id;
      const gherkinDocsForProject = this.gherkinDocuments.getDocumentsForProject(testCaseRun.projectInfo);
      const testCase = this.testCases.getOrCreate(testId, () => new _TestCase.TestCase(testId, gherkinDocsForProject));
      testCase.addRun(testCaseRun);
      testCaseRun.testCase = testCase;
    });
  }
  async loadFeatures() {
    await this.gherkinDocuments.load(this.testCaseRuns);
  }
  // private createTestCaseRuns() {
  //   this.onTestEnds.forEach(({ test, result }) => {
  //     // For skipped tests Playwright doesn't run fixtures
  //     // and we don't have bddData attachment -> don't know feature uri.
  //     // Don't add such test run to report.
  //     if (test.expectedStatus === 'skipped') return;
  //     const testCaseRun = new TestCaseRun(test, result, this.hooks);
  //     this.testCaseRuns.push(testCaseRun);
  //   });
  // }
  addMeta() {
    this.report.meta = new _Meta.Meta().buildMessage();
  }
  addSourcesAndDocuments() {
    const {
      sources,
      gherkinDocuments
    } = this.gherkinDocuments.buildMessages();
    this.report.source = sources;
    this.report.gherkinDocument = gherkinDocuments;
  }
  addPickles() {
    this.report.pickle = new _Pickles.Pickles().buildMessages(this.testCases);
  }
  addHooks() {
    this.hooks.forEach(hook => {
      const message = hook.buildMessage();
      this.report.hook.push(message);
    });
  }
  addTestCases() {
    this.testCases.forEach(testCase => {
      const message = testCase.buildMessage();
      this.report.testCase.push(message);
    });
  }
  addTestCaseRuns() {
    this.testCaseRuns.map(testCaseRun => {
      const messages = testCaseRun.buildMessages();
      this.report.testCaseRuns.push(...messages);
    });
  }
  addTestRunStarted() {
    const {
      startTime
    } = this.getFullResultTiming();
    const testRunStarted = {
      timestamp: (0, _timing.toCucumberTimestamp)(startTime.getTime())
    };
    this.report.testRunStarted = {
      testRunStarted
    };
  }
  addTestRunFinished() {
    const {
      startTime,
      duration
    } = this.getFullResultTiming();
    const testRunFinished = {
      success: this.fullResult.status === 'passed',
      timestamp: (0, _timing.toCucumberTimestamp)(startTime.getTime() + duration)
    };
    this.report.testRunFinished = {
      testRunFinished
    };
  }
  buildEventDataCollector() {
    this.emitMessages(this.eventDataCollectorEmitter);
  }
  getFullResultTiming() {
    if (this.fullResultTiming) {
      return this.fullResultTiming;
    }
    // result.startTime and result.duration were added in pw 1.37
    // see: https://github.com/microsoft/playwright/pull/26760
    if ('startTime' in this.fullResult && 'duration' in this.fullResult) {
      this.fullResultTiming = {
        startTime: this.fullResult.startTime,
        duration: this.fullResult.duration
      };
    } else {
      // Calculate overall startTime and duration based on test timings
      const items = this.testCaseRuns.map(t => t.result);
      this.fullResultTiming = (0, _timing.calcMinMaxByArray)(items);
    }
    return this.fullResultTiming;
  }
}
exports.MessagesBuilder = MessagesBuilder;