"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GherkinDocuments = void 0;
var _AutofillMap = require("../../../utils/AutofillMap");
var _loadFeatures = require("../../../cucumber/loadFeatures");
var _configDir = require("../../../config/configDir");
var _GherkinDocumentClone = require("./GherkinDocumentClone");
var _GherkinDocument = require("./GherkinDocument");
var _Projects = require("./Projects");
var _env = require("../../../config/env");
var _lang = require("../../../config/lang");
class GherkinDocuments {
  featuresLoader = new _loadFeatures.FeaturesLoader();
  projectsPerFeaturePath = new _AutofillMap.AutofillMap();
  gherkinDocumentsPerProject = new _AutofillMap.AutofillMap();
  constructor() {}
  async load(testCaseRuns) {
    this.fillProjectsPerFeaturePath(testCaseRuns);
    const cwd = (0, _configDir.getPlaywrightConfigDir)();
    const featurePaths = [...this.projectsPerFeaturePath.keys()];
    await this.featuresLoader.load(featurePaths, {
      relativeTo: cwd,
      defaultDialect: this.getFeaturesLang()
    });
    this.fillGherkinDocumentsPerProject();
  }
  getDocumentsForProject(projectInfo) {
    const docs = this.gherkinDocumentsPerProject.get(projectInfo);
    if (!docs) {
      throw new Error(`No gherkin docs for project ${projectInfo === null || projectInfo === void 0 ? void 0 : projectInfo.projectName}`);
    }
    return docs;
  }
  buildMessages() {
    const sources = [];
    const gherkinDocuments = [];
    this.gherkinDocumentsPerProject.forEach((docs, project) => {
      docs.forEach(doc => {
        sources.push(this.buildSourceMessage(project, doc));
        gherkinDocuments.push(new _GherkinDocument.GherkinDocumentMessage(project, doc).build());
      });
    });
    return {
      sources,
      gherkinDocuments
    };
  }
  fillProjectsPerFeaturePath(testCaseRuns) {
    testCaseRuns.forEach(testCaseRun => {
      const projects = this.projectsPerFeaturePath.getOrCreate(testCaseRun.bddData.uri, () => new Set());
      projects.add(testCaseRun.projectInfo);
    });
  }
  fillGherkinDocumentsPerProject() {
    this.featuresLoader.getDocumentsWithPickles().forEach(gherkinDocument => {
      if (!gherkinDocument.uri) {
        throw new Error(`Feature without uri`);
      }
      const projects = this.projectsPerFeaturePath.get(gherkinDocument.uri);
      if (!projects) {
        throw new Error(`Feature without projects: ${gherkinDocument.uri}`);
      }
      projects.forEach(project => {
        this.addGherkinDocumentToProject(project, gherkinDocument);
      });
    });
  }
  addGherkinDocumentToProject(projectInfo, gherkinDocument) {
    const projectDocs = this.gherkinDocumentsPerProject.getOrCreate(projectInfo, () => []);
    const clonedDocument = new _GherkinDocumentClone.GherkinDocumentClone(gherkinDocument).getClone();
    projectDocs.push(clonedDocument);
  }
  buildSourceMessage(projectInfo, doc) {
    if (!doc.uri) {
      throw new Error(`Doc without uri`);
    }
    const originalSource = this.featuresLoader.gherkinQuery.getSource(doc.uri);
    if (!originalSource) {
      throw new Error(`No source`);
    }
    const source = {
      ...originalSource,
      uri: (0, _Projects.getFeatureUriWithProject)(projectInfo, doc.uri)
    };
    return {
      source
    };
  }
  getFeaturesLang() {
    const langsSet = new Set();
    const envConfigs = (0, _env.getEnvConfigs)();
    Object.values(envConfigs).forEach(config => {
      langsSet.add(config.language || _lang.LANG_EN);
    });
    const langs = [...langsSet];
    if (langs.length > 1) {
      throw new Error([`Multi-language features are not supported yet.`, `Detected languages: ${langs.join(', ')}`].join(' '));
    }
    return langs[0];
  }
}
exports.GherkinDocuments = GherkinDocuments;