"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TestCase = void 0;
var _utils = require("../../../utils");
class TestCase {
  id;
  gherkinDocuments;
  #pickle;
  #projectInfo;
  beforeHooks = new Map();
  afterHooks = new Map();
  mainSteps = [];
  constructor(id, gherkinDocuments) {
    this.id = id;
    this.gherkinDocuments = gherkinDocuments;
  }
  get projectInfo() {
    if (!this.#projectInfo) {
      throw new Error(`Empty projectInfo for testCase: ${this.id}`);
    }
    return this.#projectInfo;
  }
  get pickle() {
    if (!this.#pickle) {
      throw new Error(`Empty pickle for testCase: ${this.id}`);
    }
    return this.#pickle;
  }
  addRun(testCaseRun) {
    if (!this.#projectInfo) {
      this.#projectInfo = testCaseRun.projectInfo;
    }
    this.addHooks(testCaseRun, 'before');
    this.addHooks(testCaseRun, 'after');
    if (!this.#pickle) {
      this.#pickle = this.findPickle(testCaseRun.bddData);
      this.addStepsFromPickle(this.#pickle);
    }
    this.addStepsArgumentsLists(testCaseRun);
  }
  getHooks(hookType) {
    return hookType == 'before' ? this.beforeHooks : this.afterHooks;
  }
  getMainSteps() {
    return this.mainSteps;
  }
  buildMessage() {
    const testSteps = [...Array.from(this.beforeHooks.values()).map(hook => hook.testStep), ...(this.mainSteps || []), ...Array.from(this.afterHooks.values()).map(hook => hook.testStep)];
    const testCase = {
      id: this.id,
      pickleId: this.pickle.id,
      testSteps
    };
    return {
      testCase
    };
  }
  /**
     * We collect hooks from all runs of this test case, avoiding duplicates.
     */
  addHooks(testCaseRun, hookType) {
    const testCaseHooks = hookType === 'before' ? this.beforeHooks : this.afterHooks;
    const testRunHooks = testCaseRun.getExecutedHooks(hookType);
    testRunHooks.forEach(({
      hook
    }) => {
      if (testCaseHooks.has(hook.internalId)) {
        return;
      }
      const testStep = {
        id: `${this.id}-${hookType}-${testCaseHooks.size}`,
        hookId: hook.id
      };
      testCaseHooks.set(hook.internalId, {
        hook,
        testStep
      });
    });
  }
  /**
     * Initially create steps from pickle steps, with empty stepMatchArgumentsLists.
     */
  addStepsFromPickle(pickle) {
    this.mainSteps = pickle.steps.map((pickleStep, stepIndex) => {
      return {
        id: `${this.id}-step-${stepIndex}`,
        pickleStepId: pickleStep.id,
        stepDefinitionIds: [],
        stepMatchArgumentsLists: []
      };
    });
  }
  /**
     * Fill stepMatchArgumentsLists from all test runs.
     * It allows to fill as many steps as possible.
     * Possibly, we write the same stepMatchArgumentsLists several times,
     * looks like it's not a problem as they should be equal for all runs.
     */
  addStepsArgumentsLists(testCaseRun) {
    testCaseRun.bddData.steps.forEach((bddDataStep, stepIndex) => {
      var _this$mainSteps;
      const testCaseStep = (_this$mainSteps = this.mainSteps) === null || _this$mainSteps === void 0 ? void 0 : _this$mainSteps[stepIndex];
      if (testCaseStep && bddDataStep.stepMatchArgumentsLists) {
        testCaseStep.stepMatchArgumentsLists = bddDataStep.stepMatchArgumentsLists;
      }
    });
  }
  findPickle({
    uri,
    pickleLocation
  }) {
    const doc = this.gherkinDocuments.find(doc => doc.uri === uri);
    if (!doc) {
      throw new Error('GherkinDocument not found');
    }
    const pickle = doc.pickles.find(pickle => {
      return (0, _utils.stringifyLocation)(pickle.location) === pickleLocation;
    });
    if (!pickle) {
      throw new Error('Pickle not found');
    }
    return pickle;
  }
}
exports.TestCase = TestCase;