"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TestCaseRun = void 0;
var _utils = require("../../../utils");
var _TestStepRun = require("./TestStepRun");
var _timing = require("./timing");
var _pwStepUtils = require("./pwStepUtils");
var _AttachmentMapper = require("./AttachmentMapper");
var _TestCaseRunHooks = require("./TestCaseRunHooks");
var _Projects = require("./Projects");
var _bddData = require("../../../run/bddData");
class TestCaseRun {
  test;
  result;
  hooks;
  id;
  bddData;
  testCase;
  attachmentMapper;
  projectInfo;
  // collect steps with error and show only these errors in report.
  // it allows to not show the same error on parent steps
  errorSteps = new Set();
  timeoutedStep;
  executedBeforeHooks;
  executedAfterHooks;
  executedSteps;
  // eslint-disable-next-line max-params
  constructor(test, result, hooks) {
    this.test = test;
    this.result = result;
    this.hooks = hooks;
    this.id = this.generateTestRunId();
    this.bddData = this.extractBddData();
    this.projectInfo = (0, _Projects.getProjectInfo)(this.test);
    this.attachmentMapper = new _AttachmentMapper.AttachmentMapper(this.result);
    this.executedSteps = this.fillExecutedSteps();
    this.executedBeforeHooks = this.fillExecutedHooks('before');
    this.executedAfterHooks = this.fillExecutedHooks('after');
  }
  getTestCase() {
    if (!this.testCase) {
      throw new Error(`TestCase is not set.`);
    }
    return this.testCase;
  }
  isTimeouted() {
    return this.result.status === 'timedOut';
  }
  generateTestRunId() {
    return `${this.test.id}-attempt-${this.result.retry}`;
  }
  extractBddData() {
    const {
      bddData,
      annotationIndex
    } = (0, _bddData.getBddDataFromTest)(this.test);
    if (!bddData) {
      throw new Error(`__bddData annotation is not found for test "${this.test.title}".`);
    }
    // remove __bddData annotation from test (mutate)
    this.test.annotations.splice(annotationIndex, 1);
    return bddData;
  }
  fillExecutedSteps() {
    const possiblePwSteps = this.getPossiblePlaywrightBddSteps();
    return this.bddData.steps.map(bddDataStep => {
      const pwStep = this.findPlaywrightStep(possiblePwSteps, bddDataStep);
      this.registerErrorStep(pwStep);
      this.registerTimeoutedStep(pwStep);
      return {
        bddDataStep,
        pwStep
      };
    });
  }
  fillExecutedHooks(hookType) {
    return new _TestCaseRunHooks.TestCaseRunHooks(this, hookType).fill(this.executedSteps);
  }
  registerErrorStep(pwStep) {
    if (pwStep !== null && pwStep !== void 0 && pwStep.error) {
      this.errorSteps.add(pwStep);
    }
  }
  // eslint-disable-next-line complexity
  registerTimeoutedStep(pwStep) {
    if (!pwStep || !this.isTimeouted() || this.timeoutedStep) {
      return;
    }
    const {
      error
    } = pwStep;
    if ((0, _pwStepUtils.isUnknownDuration)(pwStep) || this.result.errors.some(e => e.message === (error === null || error === void 0 ? void 0 : error.message))) {
      this.timeoutedStep = pwStep;
    }
  }
  buildMessages() {
    return [this.buildTestCaseStarted(), ...this.executedBeforeHooks.buildMessages(), ...this.buildStepRuns(), ...this.executedAfterHooks.buildMessages(), this.buildTestCaseFinished()];
  }
  getExecutedHooks(hookType) {
    return hookType === 'before' ? this.executedBeforeHooks.executedHooks : this.executedAfterHooks.executedHooks;
  }
  buildTestCaseStarted() {
    const testCaseStarted = {
      id: this.id,
      attempt: this.result.retry,
      testCaseId: this.getTestCase().id,
      // workerId: 'worker-1'
      timestamp: (0, _timing.toCucumberTimestamp)(this.result.startTime.getTime())
    };
    return {
      testCaseStarted
    };
  }
  buildStepRuns() {
    return this.getTestCase().getMainSteps().reduce((messages, testStep, stepIndex) => {
      const {
        pwStep
      } = this.executedSteps[stepIndex] || {};
      const testStepRun = new _TestStepRun.TestStepRun(this, testStep, pwStep);
      return messages.concat(testStepRun.buildMessages());
    }, []);
  }
  buildTestCaseFinished() {
    const {
      startTime,
      duration
    } = this.result;
    const testCaseFinished = {
      testCaseStartedId: this.id,
      willBeRetried: Boolean(this.result.error && this.result.retry < this.test.retries),
      timestamp: (0, _timing.toCucumberTimestamp)(startTime.getTime() + duration)
    };
    return {
      testCaseFinished
    };
  }
  findPlaywrightStep(possiblePwSteps, bddDataStep) {
    return possiblePwSteps.find(pwStep => {
      return pwStep.location && (0, _utils.stringifyLocation)(pwStep.location) === bddDataStep.pwStepLocation;
    });
  }
  getPossiblePlaywrightBddSteps() {
    // Before we collected only top-level steps and steps from before hooks (as they are background)
    // But it's more reliable to just collect all test.step items b/c some Playwright versions
    // move steps to fixtures (see: https://github.com/microsoft/playwright/issues/30075)
    // Collecting all test.step items should be ok, as later we anyway map them by location.
    return (0, _pwStepUtils.collectStepsWithCategory)(this.result, 'test.step');
  }
}
exports.TestCaseRun = TestCaseRun;