"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Fixture = Fixture;
exports.getPomNodeByFixtureName = getPomNodeByFixtureName;
var _steps = require("./steps");
var _exit = require("../../utils/exit");
/**
 * Class level @Fixture decorator.
 */

/**
 * Graph of POM class inheritance.
 * Allows to guess correct fixture by step text.
 */
const pomGraph = new Map();
/**
 * @Fixture decorator.
 */
function Fixture(fixtureName) {
  // context parameter is required for decorator by TS even though it's not used
  return Ctor => {
    createPomNode(Ctor, fixtureName);
  };
}
function createPomNode(Ctor, fixtureName) {
  const pomNode = {
    fixtureName,
    className: Ctor.name,
    children: new Set()
  };
  ensureUniqueFixtureName(pomNode);
  pomGraph.set(Ctor, pomNode);
  (0, _steps.linkStepsWithPomNode)(Ctor, pomNode);
  linkParentWithPomNode(Ctor, pomNode);
  return pomNode;
}
function ensureUniqueFixtureName({
  fixtureName,
  className
}) {
  if (!fixtureName) {
    return;
  }
  const existingPom = getPomNodeByFixtureName(fixtureName);
  if (existingPom) {
    (0, _exit.exit)(`Duplicate fixture name "${fixtureName}"`, `defined for classes: ${existingPom.className}, ${className}`);
  }
}
function linkParentWithPomNode(Ctor, pomNode) {
  const parentCtor = Object.getPrototypeOf(Ctor);
  if (!parentCtor) {
    return;
  }
  // if parentCtor is not in pomGraph, add it.
  // Case: parent class is not marked with @Fixture, but has decorator steps (base class)
  const parentPomNode = pomGraph.get(parentCtor) || createPomNode(parentCtor, '');
  parentPomNode.children.add(pomNode);
}
function getPomNodeByFixtureName(fixtureName) {
  for (const pomNode of pomGraph.values()) {
    if (pomNode.fixtureName === fixtureName) {
      return pomNode;
    }
  }
}